"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// builtin
const node_process_1 = require("node:process");
// external
const caterpillar_1 = require("caterpillar");
// local
const index_js_1 = require("./index.js");
// cli
const argument_1 = __importDefault(require("@bevry/argument"));
const help = `
	USAGE:
	projectz [...options]

	OPTIONS:
	--path=<string>
	  The directory to process, defaults to the current working directory.

	--[no-]verbose[=<boolean>]
	  Set the logging level to verbose (log level 7 instead of the default 6).

	--[no-]offline[=<boolean>]
	  If enabled, then remote updates will be not performed (such as fetching latest backers).

	compile
	  Prior to version 3.5, this was necessary for projectz to do its thing. Now it is the default behaviour.`;
async function main(args) {
    // parse arguments
    let path = (0, node_process_1.cwd)(), verbose = false, offline = false;
    for (const arg of args) {
        const a = new argument_1.default(arg);
        switch (a.key) {
            case 'help': {
                return argument_1.default.help(help);
            }
            case 'path': {
                path = a.string({ enabled: path });
                break;
            }
            case 'verbose': {
                verbose = a.boolean();
                break;
            }
            case 'offline': {
                offline = a.boolean();
                break;
            }
            case 'compile': {
                console.warn(a.arg, 'is no longer necessary, and is now the default behaviour.');
                break;
            }
            default: {
                switch (a.value) {
                    case 'compile': {
                        console.warn(a.arg, 'is no longer necessary, and is now the default behaviour.');
                        break;
                    }
                    default: {
                        return a.unknown();
                    }
                }
            }
        }
    }
    // setup our logging
    const level = verbose ? 7 : 6;
    const logger = new caterpillar_1.Logger({ lineLevel: verbose ? level : -Infinity });
    const filter = new caterpillar_1.Filter({ filterLevel: level });
    const human = new caterpillar_1.Human();
    logger.pipe(filter).pipe(human).pipe(node_process_1.stdout);
    // configure projectz
    const project = new index_js_1.Projectz({
        offline,
        cwd: path,
        log: logger.log.bind(logger),
    });
    // execute
    await project.compile();
    // done
    logger.log('info', 'Completed successfully');
}
main(node_process_1.argv.slice(2)).catch(argument_1.default.catch(help));
